@@ -3,13 +3,13 @@ package Kelp::Routes;
 use Carp;
 
 use Kelp::Base;
+use Kelp::Routes::Pattern;
 use Plack::Util;
 use Class::Inspector;
 
-attr base        => '';
-attr pattern_obj => 'Kelp::Routes::Pattern';
-attr routes      => sub { [] };
-attr names       => sub { {} };
+attr base    => '';
+attr routes  => sub { [] };
+attr names   => sub { {} };
 
 # Cache
 attr _CACHE => sub { {} };
@@ -119,7 +119,7 @@ sub _parse_route {
     }
 
     # Create pattern object
-    push @{ $self->routes }, $self->build_pattern( $val );
+    push @{ $self->routes }, Kelp::Routes::Pattern->new(%$val);
 
     # Add route index to names
     if ( my $name = $val->{name} ) {
@@ -135,14 +135,6 @@ sub _parse_route {
     }
 }
 
-# Override to use a custom pattern object
-sub build_pattern {
-    my ( $self, $args ) = @_;
-    my $package = $self->pattern_obj;
-    eval qq{require $package};
-    return $package->new( %$args );
-}
-
 sub url {
     my $self = shift;
     my $name = shift // die "Route name is missing";
@@ -179,11 +171,12 @@ sub dispatch {
     $route || die "No route pattern instance supplied";
 
     # Shortcuts
+    my $req = $app->req;
     my $to  = $route->to;
 
     # Destination must be either a scalar, or a code reference
-    if ( !$to || ( ref $to && ref $to ne 'CODE' ) ) {
-        die 'Invalid destination for ' . $app->req->path;
+    if ( !$to || ref $to && ref $to ne 'CODE' ) {
+        die 'Invalid destination for ' . $req->path;
     }
 
     # If the destination is not a code reference, then we assume it's
@@ -192,14 +185,13 @@ sub dispatch {
 
         # Check if the destination function exists
         unless ( exists &$to ) {
-            die sprintf( 'Route not found %s for %s', $to, $app->req->path );
+            die sprintf( 'Route not found %s for %s', $to, $req->path );
         }
 
         # Move to reference
         $to = \&{$to};
     }
 
-    $app->before_dispatch( $route->to );
     return $to->( $app, @{ $route->param } );
 }
 
@@ -695,4 +687,3 @@ the application instance into a controller class.
 This module was inspired by L<Routes::Tiny>.
 
 =cut
-
